@@ -729,7 +729,7 @@ SPF_c_parse_ip4(SPF_response_t *spf_response, SPF_mech_t *mech, char const *star
 	const char			*end;
 	const char			*p;
 
-	char				 buf[ INET_ADDRSTRLEN ];
+	char				 buf[ INET6_ADDRSTRLEN ];
 	size_t				 len;
 	SPF_errcode_t		 err;
 
@@ -778,7 +778,7 @@ SPF_c_parse_ip6(SPF_response_t *spf_response, SPF_mech_t *mech, char const *star
 	const char			*end;
 	const char			*p;
 
-	char				 buf[ INET_ADDRSTRLEN ];
+	char				 buf[ INET6_ADDRSTRLEN ];
 	size_t				 len;
 	int					 err;
 
@@ -1188,7 +1188,7 @@ SPF_record_compile(SPF_server_t *spf_server,
 	SPF_errcode_t		 err;
 	
 	const char			*name_start;
-	int					 name_len;
+	size_t				 name_len;
 
 	const char			*val_start;
 	const char			*val_end;
