@@ -58,8 +58,15 @@
 #include "spf_dns_cache.h"
 
 
-/*
- * this is really little more than a proof-of-concept cache.
+/**
+ * @file
+ *
+ * Implements a simple cache using a list hash. There is no reclaim
+ * list, since GNU malloc has clue.
+ *
+ * This original description from Wayne is no longer true:
+ *
+ * This is really little more than a proof-of-concept cache.
  *
  * The cache size is fixed and uses the CRC-32 function as a hash
  * generator.  Little is done about hash collisions, no alternate hash
@@ -284,7 +291,7 @@ SPF_dns_cache_bucket_find(SPF_dns_cache_config_t *spfhook,
 }
 
 /* This must be called with the lock held. */
-static void
+static SPF_errcode_t
 SPF_dns_cache_bucket_add(SPF_dns_cache_config_t *spfhook,
 				SPF_dns_rr_t *rr, int idx)
 {
@@ -292,13 +299,19 @@ SPF_dns_cache_bucket_add(SPF_dns_cache_config_t *spfhook,
 
 	bucket = (SPF_dns_cache_bucket_t *)
 				malloc(sizeof(SPF_dns_cache_bucket_t));
+	if (! bucket)
+		return SPF_E_NO_MEMORY;
 	bucket->next = spfhook->cache[idx];
 	spfhook->cache[idx] = bucket;
 	bucket->rr = rr;
+	return SPF_E_SUCCESS;
 }
 
 
-static void
+/**
+ * Patches up an rr for insertion into the cache.
+ */
+static SPF_errcode_t
 SPF_dns_cache_rr_fixup(SPF_dns_cache_config_t *spfhook,
 				SPF_dns_rr_t *cached_rr,
 				const char *domain, ns_type rr_type)
@@ -312,17 +325,12 @@ SPF_dns_cache_rr_fixup(SPF_dns_cache_config_t *spfhook,
 	/* XXX I'm still not sure about this bit. */
 	if (cached_rr->domain == NULL || cached_rr->domain[0] != '\0') {
 		char	*new_domain;
-		size_t	new_len = strlen( domain ) + 1;
+		size_t	 new_len = strlen(domain) + 1;
 
-		if ( cached_rr->domain_buf_len < new_len ) {
+		if (cached_rr->domain_buf_len < new_len) {
 			new_domain = realloc(cached_rr->domain, new_len);
-			if ( new_domain == NULL ) {
-				/* XXX Panic! */
-				// SPF_dns_rr_free(cached_rr);
-				// spfhook->cache[ h ] = NULL;
-				// return fetched_rr;
-			}
-
+			if (new_domain == NULL)
+				return SPF_E_NO_MEMORY;
 			cached_rr->domain = new_domain;
 			cached_rr->domain_buf_len = new_len;
 		}
@@ -348,9 +356,14 @@ SPF_dns_cache_rr_fixup(SPF_dns_cache_config_t *spfhook,
     }
 
 	cached_rr->utc_ttl = cached_rr->ttl + time(NULL);
+
+	return SPF_E_SUCCESS;
 }
 
 
+/**
+ * Can return NULL on out-of-memory condition.
+ */
 static SPF_dns_rr_t *
 SPF_dns_cache_lookup(SPF_dns_server_t *spf_dns_server,
 				const char *domain, ns_type rr_type, int should_cache)
@@ -363,17 +376,23 @@ SPF_dns_cache_lookup(SPF_dns_server_t *spf_dns_server,
 
 	spfhook = SPF_voidp2spfhook(spf_dns_server->hook);
 
-    pthread_mutex_lock(&(spfhook->cache_lock));
-
+	/* max_hash_len and cache_size are constant, so this be done
+	 * outside the lock. */
 	idx = hash(spfhook, domain, 0 /* spfhook->hash_mask+rr_type */);
 	idx &= (spfhook->cache_size - 1);
 
+    pthread_mutex_lock(&(spfhook->cache_lock));
+
 	bucket = SPF_dns_cache_bucket_find(spfhook, domain, rr_type, idx);
 	if (bucket != NULL) {
 		if (bucket->rr != NULL) {
-			SPF_dns_rr_dup(&rr, bucket->rr);
-			pthread_mutex_unlock(&(spfhook->cache_lock));
-			return rr;
+			if (SPF_dns_rr_dup(&rr, bucket->rr) == SPF_E_SUCCESS) {
+				pthread_mutex_unlock(&(spfhook->cache_lock));
+				return rr;
+			}
+			else if (rr != NULL) {
+				SPF_dns_rr_free(rr);	/* Within the lock. :-( */
+			}
 		}
 	}
 
@@ -383,164 +402,33 @@ SPF_dns_cache_lookup(SPF_dns_server_t *spf_dns_server,
 
 	pthread_mutex_unlock(&(spfhook->cache_lock));
 
-    if ( spf_dns_server->layer_below )
-		rr = SPF_dns_lookup( spf_dns_server->layer_below,
-						domain, rr_type, should_cache );
-    else
+    if (!spf_dns_server->layer_below)
 		return SPF_dns_rr_new_nxdomain(spf_dns_server, domain);
 
+	rr = SPF_dns_lookup( spf_dns_server->layer_below,
+					domain, rr_type, should_cache );
     if (spfhook->conserve_cache && !should_cache)
 		return rr;
 
     pthread_mutex_lock(&(spfhook->cache_lock));
 
-	if (SPF_dns_rr_dup(&cached_rr, rr))	{
-		/* XXX This represents a malloc failure. Put a warning in. */
-		pthread_mutex_unlock(&(spfhook->cache_lock));
-		if (cached_rr)
-			SPF_dns_rr_free(cached_rr);
-		return rr;
-	}
-
-	SPF_dns_cache_rr_fixup(spfhook, cached_rr, domain, rr_type);
-
-	SPF_dns_cache_bucket_add(spfhook, cached_rr, idx);
-
-    pthread_mutex_unlock(&(spfhook->cache_lock));
-
-    return rr;
-}
-
-
-
-
-
-
-#if 0
-
-static SPF_dns_rr_t *
-SPF_dns_cache_lookup(SPF_dns_server_t *spf_dns_server,
-				const char *domain, ns_type rr_type, int should_cache)
-{
-    SPF_dns_cache_config_t	*spfhook;
-	SPF_errcode_t			 err;
-
-    SPF_dns_rr_t	*cached_rr, *fetched_rr;
-    int				h;
-    time_t			t = 0;
-    char			*p;
-
-	spfhook = SPF_voidp2spfhook( spf_dns_server->hook );
-
-    pthread_mutex_lock(&(spfhook->cache_lock));
-
-    /* see if the RR is in the cache */
-    h = hash( spfhook, domain, spfhook->hash_mask + rr_type );
-    cached_rr = spfhook->cache[ h ];
-
-    if ( cached_rr
-	  && cached_rr->rr_type == rr_type
-	  && strcmp( cached_rr->domain, domain ) == 0
-	  && cached_rr->utc_ttl >= (t = time( NULL )))
-    {
-		spfhook->hit++;
-		if ( spf_dns_server->debug > 1 )
-			SPF_debugf( "hit!  %d/%d  h: %d  should_cache: %d",
-			spfhook->hit, spfhook->miss, h, should_cache);
-		SPF_dns_rr_dup(&cached_rr, cached_rr);
-		pthread_mutex_unlock(&(spfhook->cache_lock));
-		return cached_rr;
-    }
-
-    spfhook->miss++;
-    if ( spf_dns_server->debug > 1 )
-		SPF_debugf( "miss...  %d/%d  h: %d  should_cache: %d%s",
-			spfhook->hit, spfhook->miss, h,
-			should_cache, cached_rr == NULL ? "  cold" : ""
-			);
-
-    pthread_mutex_unlock(&(spfhook->cache_lock));
-
-    if ( spf_dns_server->layer_below )
-		fetched_rr = SPF_dns_lookup( spf_dns_server->layer_below,
-						domain, rr_type, should_cache );
-    else
-		return SPF_dns_rr_new_nxdomain(spf_dns_server, domain);
-
-    pthread_mutex_lock(&(spfhook->cache_lock));
-    if ( spfhook->conserve_cache  &&  !should_cache ) {
-		pthread_mutex_unlock(&(spfhook->cache_lock));
-		return fetched_rr;
-    }
-
-    /* try to store the fetched RR into the cache */
-    if ( cached_rr != NULL ) {
-		SPF_dns_rr_free(cached_rr);
-		/* XXX Shevek put this in to clobber the entry */
-		/* I have a nasty feeling that this might actually
-		 * be the right answer. */
-		cached_rr = NULL;
-		spfhook->cache[ h ] = NULL;
-	}
-	if ( ( err = SPF_dns_rr_dup(&cached_rr, fetched_rr) ) )
-		fprintf(stderr, "Oh shit. %d\n", err);
-
-    /* make sure the RR has enough data to be useful for caching */
-    if ( cached_rr->rr_type == ns_t_any ) {
-		cached_rr->rr_type = rr_type;
-		/* Huh? Ed. */
-		if ( cached_rr->domain )
-			cached_rr->domain[0] = '\0';
-    }
-
-	if (cached_rr->domain == NULL || cached_rr->domain[0] != '\0') {
-		char   *new_domain;
-		size_t new_len = strlen( domain ) + 1;
-
-		if ( cached_rr->domain_buf_len < new_len ) {
-			new_domain = realloc( cached_rr->domain, new_len );
-			if ( new_domain == NULL ) {
-				SPF_dns_rr_free( cached_rr );
-				spfhook->cache[ h ] = NULL;
-				return fetched_rr;
+	if (SPF_dns_rr_dup(&cached_rr, rr) == SPF_E_SUCCESS) {
+		if (SPF_dns_cache_rr_fixup(spfhook, cached_rr, domain, rr_type) == SPF_E_SUCCESS){
+			if (SPF_dns_cache_bucket_add(spfhook, cached_rr, idx) == SPF_E_SUCCESS) {
+				pthread_mutex_unlock(&(spfhook->cache_lock));
+				return rr;
 			}
-
-			cached_rr->domain = new_domain;
-			cached_rr->domain_buf_len = new_len;
 		}
-		strcpy( cached_rr->domain, domain );
 	}
 
-    /* set up the ttl values */
-    if ( cached_rr->ttl < spfhook->min_ttl )
-		cached_rr->ttl = spfhook->min_ttl;
-
-    if ( cached_rr->ttl < spfhook->txt_ttl
-			&& cached_rr->rr_type == ns_t_txt )
-		cached_rr->ttl = spfhook->txt_ttl;
-
-    if ( cached_rr->ttl < spfhook->err_ttl
-			&& cached_rr->herrno != NETDB_SUCCESS )
-		cached_rr->ttl = spfhook->err_ttl;
-
-    if ( cached_rr->ttl < spfhook->rdns_ttl ) {
-		p = strstr( cached_rr->domain, ".arpa" );
-		if ( p && p[ sizeof( ".arpa" )-1 ] == '\0' )
-			cached_rr->ttl = spfhook->rdns_ttl;
-    }
-
-    if ( t == 0 )
-		t = time( NULL );
-	cached_rr->utc_ttl = cached_rr->ttl + t;
-
-    spfhook->cache[ h ] = cached_rr;
-
     pthread_mutex_unlock(&(spfhook->cache_lock));
 
-    return fetched_rr;
-}
-#endif
+	if (cached_rr)
+		SPF_dns_rr_free(cached_rr);
 
+	return rr;
+
+}
 
 
 static void
@@ -571,6 +459,7 @@ SPF_dns_cache_free( SPF_dns_server_t *spf_dns_server )
 				}
 			}
 			free(spfhook->cache);
+			spfhook->cache = NULL;
 		}
 
 		pthread_mutex_unlock(&(spfhook->cache_lock));
