@@ -505,7 +505,7 @@ SPF_i_match_ip6(SPF_server_t *spf_server,
 	char		dst_ip6_buf[ INET6_ADDRSTRLEN ];
 
 	struct in6_addr		src_ipv6;
-	int				cidr, mask;
+	int				cidr, cidr_save, mask;
 	int				i;
 	int				match;
 
@@ -517,6 +517,7 @@ SPF_i_match_ip6(SPF_server_t *spf_server,
 	cidr = SPF_i_mech_cidr(spf_request, mech);
 	if ( cidr == 0 )
 		cidr = 128;
+	cidr_save = cidr;
 
 	match = TRUE;
 	for( i = 0; i < array_elem( ipv6.s6_addr ) && match; i++ )
@@ -538,7 +539,7 @@ SPF_i_match_ip6(SPF_server_t *spf_server,
 		INET_NTOP(AF_INET6, &ipv6.s6_addr,
 							dst_ip6_buf, sizeof(dst_ip6_buf));
 		SPF_debugf( "ip_match:  %s == %s  (/%d):  %d",
-				src_ip6_buf, dst_ip6_buf, cidr, match );
+				src_ip6_buf, dst_ip6_buf, cidr_save, match );
 	}
 
 	return match;
@@ -549,8 +550,8 @@ SPF_i_match_domain(SPF_server_t *spf_server,
 				const char *hostname, const char *domain)
 {
 	const char	*hp;
-	int			 hlen;
-	int			 dlen;
+	size_t		 hlen;
+	size_t		 dlen;
 
 	if (spf_server->debug)
 		SPF_debugf( "%s ?=? %s", hostname, domain );
