@@ -42,6 +42,10 @@
 # endif
 #endif
 
+#ifdef HAVE_NETDB_H
+# include <netdb.h>
+#endif
+
 #ifndef HOST_NAME_MAX
 #define HOST_NAME_MAX 255
 #endif 
@@ -59,7 +63,7 @@
 #include "spf_dns_internal.h"
 
 
-WARN_UNUSED_RESULT
+__attribute__((warn_unused_result))
 static SPF_errcode_t
 SPF_server_set_rec_dom_ghbn(SPF_server_t *sp)
 {
@@ -136,6 +140,7 @@ SPF_server_new(SPF_server_dnstype_t dnstype, int debug)
 	if (! sp)
 		return sp;
 	SPF_server_new_common_pre(sp, debug);
+	sp->destroy_resolver = 1;
 
 	switch (dnstype) {
 		case SPF_DNS_RESOLV:
@@ -180,6 +185,7 @@ SPF_server_new_dns(SPF_dns_server_t *dns, int debug)
 	if (! sp)
 		return sp;
 	SPF_server_new_common_pre(sp, debug);
+	sp->destroy_resolver = 0;
 	sp->resolver = dns;
 	SPF_server_new_common_post(sp);
 	return sp;
@@ -193,7 +199,7 @@ SPF_server_new_dns(SPF_dns_server_t *dns, int debug)
 void
 SPF_server_free(SPF_server_t *sp)
 {
-	if (sp->resolver)	// Never false, but let's be defensive.
+	if (sp->resolver && sp->destroy_resolver)
 		SPF_dns_free(sp->resolver);
 	if (sp->local_policy)
 		SPF_record_free(sp->local_policy);
@@ -427,14 +433,15 @@ retry:
 			SPF_debugf("Comparing '%s' with '%s'",
 					SPF_VER_STR " ", rr_txt->rr[i]->txt);
 		*/
-		if (strncasecmp( rr_txt->rr[i]->txt,
-					  SPF_VER_STR " ", sizeof( SPF_VER_STR " " ) - 1) == 0)
-		{
-			if ( spf_server->debug > 0 )
-				SPF_debugf( "found SPF record: %s", rr_txt->rr[i]->txt );
-
-			num_found++;
-			idx_found = i;
+		if (strncasecmp(rr_txt->rr[i]->txt,
+					  SPF_VER_STR, sizeof(SPF_VER_STR) - 1) == 0) {
+			char	e = rr_txt->rr[i]->txt[sizeof(SPF_VER_STR) - 1];
+			if (e == ' ' || e == '\0') {
+				if (spf_server->debug > 0)
+					SPF_debugf("found SPF record: %s", rr_txt->rr[i]->txt);
+				num_found++;
+				idx_found = i;
+			}
 		}
 	}
 
