@@ -174,9 +174,7 @@ unimplemented(const char flag)
 	struct option	*opt;
 	int				 i;
 
-	i = 0;
-	opt = &long_options[i];
-	while (opt->name) {
+	for (i = 0; (opt = &long_options[i])->name; i++) {
 		if (flag == opt->val) {
 			fprintf(stderr, "Unimplemented option: -%s or -%c\n",
 							opt->name, flag);
@@ -211,42 +209,42 @@ help()
 	"spfquery [control options | data options] ...\n"
 	"\n"
 	"Valid data options are:\n"
-	"	-file <filename>		   read spf data from a file.  Use '-'\n"
-	"							   to read from stdin.\n"
+	"    -file <filename>           read spf data from a file.  Use '-'\n"
+	"                               to read from stdin.\n"
 	"\n"
-	"	-ip <IP address>		   The IP address that is sending email\n"
-	"	-sender <email address>	The email address used as the\n"
-	"							   envelope-from.  If no username (local\n"
-	"							   part) is given, 'postmaster' will be\n"
-	"							   assumed.\n"
-	"	-helo <domain name>		The domain name given on the SMTP HELO\n"
-	"							   command.  This is only needed if the\n"
-	"							   -sender option is not given.\n"
-	"	-rcpt-to <email addresses> A comma separated lists of email addresses\n"
-	"							   that will have email from their secondary\n"
-	"							   MXes automatically allowed.\n"
+	"    -ip <IP address>           The IP address that is sending email\n"
+	"    -sender <email address>    The email address used as the\n"
+	"                               envelope-from.  If no username (local\n"
+	"                               part) is given, 'postmaster' will be\n"
+	"                               assumed.\n"
+	"    -helo <domain name>        The domain name given on the SMTP HELO\n"
+	"                               command.  This is only needed if the\n"
+	"                               -sender option is not given.\n"
+	"    -rcpt-to <email addresses> A comma separated lists of email addresses\n"
+	"                               that will have email from their secondary\n"
+	"                               MXes automatically allowed.\n"
 	"\n"
 	"The data options are required.  The -file option conflicts with all\n"
 	"the other data options.  The -helo and -rcpt-to are optional.\n"
 	"\n"
 	"\n"
 	"Valid control options are:\n"
-	"	-debug [debug level]	   debug level.\n"
-	"	-local <SPF mechanisms>	Local policy for whitelisting.\n"
-	"	-trusted <0|1>			 Should trusted-forwarder.org be checked?\n"
-	"	-guess <SPF mechanisms>	Default checks if no SPF record is found.\n"
-	"	-default-explanation <str> Default explanation string to use.\n"
-	"	-max-lookup <number>	   Maximum number of DNS lookups to allow\n"
-	"	-sanitize <0|1>			Clean up invalid characters in output?\n"
-	"	-name <domain name>		The name of the system doing the SPF\n"
-	"							   checking\n"
-	"	-override <...>			Override SPF records for domains\n"
-	"	-fallback <...>			Fallback SPF records for domains\n"
+	"    -debug [debug level]       debug level.\n"
+	"    -local <SPF mechanisms>    Local policy for whitelisting.\n"
+	"    -trusted <0|1>             Should trusted-forwarder.org be checked?\n"
+	"    -guess <SPF mechanisms>    Default checks if no SPF record is found.\n"
+	"    -default-explanation <str> Default explanation string to use.\n"
+	"    -max-lookup <number>       Maximum number of DNS lookups to allow\n"
+	"    -sanitize <0|1>            Clean up invalid characters in output?\n"
+	"    -name <domain name>        The name of the system doing the SPF\n"
+	"                               checking\n"
+	"    -override <...>            Override SPF records for domains\n"
+	"    -fallback <...>            Fallback SPF records for domains\n"
 	"\n"
-	"	-keep-comments			 Print comments found when reading\n"
-	"							   from a file.\n"
-	"	-version				   Print version of spfquery.\n"
-	"	-help					  Print out these options.\n"
+	"    -keep-comments             Print comments found when reading\n"
+	"                               from a file.\n"
+	"    -version                   Print version of spfquery.\n"
+	"    -help                      Print out these options.\n"
 	"\n"
 	"Examples:\n"
 	"\n"
@@ -261,7 +259,7 @@ static void
 response_print_errors(const char *context,
 				SPF_response_t *spf_response, SPF_errcode_t err)
 {
-	SPF_error_t		*spf_error;;
+	SPF_error_t		*spf_error;
 	int				 i;
 
 	printf("StartError\n");
@@ -341,6 +339,7 @@ int main( int argc, char *argv[] )
 	SPF_request_t	*spf_request = NULL;
 	SPF_response_t	*spf_response = NULL;
 	SPF_response_t	*spf_response_2mx = NULL;
+	SPF_response_t	*spf_response_fallback = NULL;
 	SPF_errcode_t	 err;
 
 	char			*opt_file = NULL;
@@ -433,7 +432,7 @@ int main( int argc, char *argv[] )
 				opts->max_lookup = atoi(optarg);
 				break;
 
-			case 'c':		/* "clean"		*/
+			case 'c':		/* "clean" */
 				opts->sanitize = atoi(optarg);
 				break;
 
@@ -604,7 +603,8 @@ int main( int argc, char *argv[] )
 
 		spf_request = SPF_request_new(spf_server);
 
-		if (SPF_request_set_ipv4_str(spf_request, req->ip)) {
+		if (SPF_request_set_ipv4_str(spf_request, req->ip)
+				&& SPF_request_set_ipv6_str(spf_request, req->ip)) {
 			printf( "Invalid IP address.\n" );
 			CONTINUE_ERROR;
 		}
@@ -670,20 +670,20 @@ int main( int argc, char *argv[] )
 		/* We now have an option to call SPF_request_query_fallback */
 		if (opts->fallback) {
 			err = SPF_request_query_fallback(spf_request,
-							&spf_response, opts->fallback);
+							&spf_response_fallback, opts->fallback);
 			if (opts->debug)
-				response_print("fallback query", spf_response_2mx);
+				response_print("fallback query", spf_response_fallback);
 			if (err) {
 				response_print_errors("Failed to query best-guess",
-								spf_response, err);
+								spf_response_fallback, err);
 				CONTINUE_ERROR;
 			}
 
 			/* append the result */
-			APPEND_RESULT(SPF_response_result(spf_response_2mx));
+			APPEND_RESULT(SPF_response_result(spf_response_fallback));
 
 			spf_response = SPF_response_combine(spf_response,
-							spf_response_2mx);
+							spf_response_fallback);
 		}
 
 		printf( "%s\n%s\n%s\n%s\n",
@@ -704,6 +704,9 @@ int main( int argc, char *argv[] )
 	FREE_REQUEST(spf_request);
 	FREE(spf_server, SPF_server_free);
 
+	FREE(req, free);
+	FREE(opts, free);
+
 #ifdef _WIN32
 	SPF_win32_cleanup();
 #endif
